/*
  Copyright (c) 2000,2001 Al Sutton (al@alsutton.com)
  All rights reserved.
  Redistribution and use in source and binary forms, with or without modification, are permitted
  provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright notice, this list of conditions
  and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright notice, this list of
  conditions and the following disclaimer in the documentation and/or other materials provided with
  the distribution.

  Neither the name of AlSutton.com nor the names of its contributors may be used to endorse or promote
  products derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package com.alsutton.filesync;

/**
 * Class holding the information about a file to be synchronized.
 */

import java.io.*;

public class FileInformation
  extends Synchronizable
{
  /**
   * The last modification time of the file.
   */

  private long lastModified;

  /**
   * The size of the file.
   */

  private long size;

  /**
   * Default constructor. Protected for internal use only.
   */

  protected FileInformation()
  {
  }

  /**
   * Constructor, stores file information.
   *
   * @param _filename The relative filename of this file.
   * @param _lastModified The last modified time of the file.
   * @param _size The size of the file.
   */

  public FileInformation( String base, File theFile )
  {
    super( base, theFile );

    lastModified = theFile.lastModified();
    size = theFile.length();
  }

  /**
   * Constructor, Reconstructs the data from strea,.
   *
   * @param stream The stream holding the data.
   */

  public void read( InputStream stream )
    throws IOException
  {
    int nextByte;

    lastModified = 0;
    for( int i = 0 ; i < 8 ; i++ )
    {
      lastModified <<= 8;

      nextByte = stream.read();
      if( nextByte == -1 )
        throw new IOException( "End of stream encountered before data finished" );

      lastModified += ((int)nextByte & 0xff );
    }

    size = 0;
    for( int i = 0 ; i < 8 ; i++ )
    {
      size <<= 8;

      nextByte = stream.read();
      if( nextByte == -1 )
        throw new IOException( "End of stream encountered before data finished" );

      size += ((int)nextByte & 0xff );
    }
    nextByte = (byte) stream.read();
    if( nextByte == -1 )
      throw new IOException( "End of stream encountered before data finished" );

    int stringLength = ((int)nextByte & 0xff );
    stringLength <<= 8;

    nextByte = (byte) stream.read();
    if( nextByte == -1 )
      throw new IOException( "End of stream encountered before data finished" );

    stringLength += ((int)nextByte & 0xff );

    int offset = 0;
    byte[] stringBuffer = new byte[ stringLength ];
    while( offset < stringLength )
    {
      offset += stream.read( stringBuffer, offset, stringLength - offset );
    }
    filename = new String( stringBuffer );
  }

  /**
   * Method to write this data to a stream.
   *
   * @param stream The stream to write it to.
   */

  public void write( OutputStream stream )
    throws IOException
  {
    byte[] data = new byte[8];

    long tempData = lastModified;
    for( int i =7 ; i >= 0 ; i-- )
    {
      data[i] = (byte)(tempData & 0xff);
      tempData >>= 8;
    }
    stream.write(data);

    tempData = size;
    for( int i = 7 ; i >= 0 ; i-- )
    {
      data[i] = (byte)(tempData & 0xff);
      tempData >>= 8;
    }
    stream.write(data);

    int stringLength = filename.length();
    data[0] = (byte)((stringLength>>8)&0xff);
    data[1] = (byte)(stringLength&0xff);
    stream.write(data, 0, 2 );

    byte[] stringData = filename.getBytes();
    stream.write(stringData);
  }

  /**
   * Method to compare this object to an object in the given base
   * directory.
   *
   * @param newBase The new base to check the file against.
   * @return true if the file is the same, false otherwise.
   */

  public boolean checkAgainstBase( String newBase )
  {
    String fullFilename = newBase + File.separatorChar + filename;
    File testFile = new File( fullFilename );

    if( testFile.exists() == false )
      return false;

    long testLastModified = testFile.lastModified();
    if( testLastModified != lastModified )
      return false;

    long testSize = testFile.length();
    if( testSize != size )
      return false;

    return true;
  }

  /**
   * Method to get the file from a specified base and send it over a stream.
   *
   * @param base The base for the file to be retrieved.
   * @param stream The stream over which it should be sent.
   */

  public void sendData( String base, OutputStream stream )
    throws IOException
  {
    String fullFilename = base + File.separatorChar + filename;
    FileInputStream theFile = new FileInputStream( fullFilename );

    byte[] buffer = new byte[4096];
    byte[] sizeData = new byte[2];
    long testCounter = size;
    while( testCounter > 0 )
    {
      int read = theFile.read(buffer);

      stream.write( (read>>8)& 0xff );
      stream.write(  read    & 0xff );

      stream.write( buffer, 0, read );
      testCounter -= read;
    }

    theFile.close();
  }

  /**
   * Method to receive and create the file in a new base area.
   *
   * @param base The new base area to receive it into.
   * @param stream The stream over which to accept the file.
   */

  public void receiveData( String base, InputStream stream )
    throws IOException
  {
    String fullFilename = base + File.separatorChar + filename;

    File theFile = new File( fullFilename );
    theFile.delete();

    FileOutputStream fileStream = new FileOutputStream( fullFilename );

    byte[] buffer = new byte[4096];
    long testCounter = size;
    while( testCounter > 0 )
    {
      int dataSize = stream.read() << 8;
      dataSize += stream.read();

      int counter = 0;
      while( counter < dataSize )
      {
        counter += stream.read(buffer, counter, dataSize-counter);
      }

      fileStream.write( buffer, 0, dataSize );
      testCounter -= dataSize;
    }

    fileStream.close();

    theFile = new File( fullFilename );
    theFile.setLastModified( lastModified );
  }
}