/*
  Copyright (c) 2000,2001 Al Sutton (al@alsutton.com)
  All rights reserved.
  Redistribution and use in source and binary forms, with or without modification, are permitted
  provided that the following conditions are met:

  1. Redistributions of source code must retain the above copyright notice, this list of conditions
  and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright notice, this list of
  conditions and the following disclaimer in the documentation and/or other materials provided with
  the distribution.

  Neither the name of AlSutton.com nor the names of its contributors may be used to endorse or promote
  products derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package com.alsutton.filesync;

import java.io.*;

/**
 * Class for reading and writing File and Directory information information
 * over streams.
 */

public class InformationHandler
{

  /**
   * The end of transmittion identifier.
   */

  public static final int END_OF_TRANSMITTION = 0;

  /**
   * The type block identifier for a file.
   */

  public static final int FILE_IDENTIFIER = 1;

  /**
   * The type block identifier for a directory
   */

  private static final int DIRECTORY_IDENTIFIER = 2;

  /**
   * The method to get the data from a stream.
   *
   * @param stream The stream to get the data from.
   * @return A file information block holding the details of the file.
   */

  public static Synchronizable getData( String base, InputStream inputStream,
    OutputStream outputStream )
    throws IOException
  {
    int dataByte = inputStream.read();
    if( dataByte == -1 )
      return null;

    Synchronizable data = null;
    switch( dataByte )
    {
      case FILE_IDENTIFIER:
        data = new FileInformation( );
        break;

      case DIRECTORY_IDENTIFIER:
        data = new DirectoryInformation( );
        break;

      case END_OF_TRANSMITTION:
        outputStream.write( END_OF_TRANSMITTION );
        return null;

      default:
        System.out.println( "Unknown object type in stream" );
        System.exit(-1);
    }

    data.read(inputStream);

    boolean checkMatch = data.checkAgainstBase( base );
    if( checkMatch == false )
    {
      outputStream.write( 0 );
      data.receiveData( base, inputStream );
    }
    else
    {
      outputStream.write( 1 );
    }

    return data;
  }

  /**
   * The method to ensure a file is synchronized.
   *
   * @param stream The stream to get the data from.
   * @return A file information block holding the details of the file.
   */

  public static void writeData( String base, File theFile,
    InputStream inputStream, OutputStream outputStream )
    throws IOException
  {
    Synchronizable data = null;
    if      ( theFile.isDirectory() )
    {
      outputStream.write( DIRECTORY_IDENTIFIER );
      data = new DirectoryInformation( base, theFile );
    }
    else if ( theFile.isFile() )
    {
      outputStream.write( FILE_IDENTIFIER );
      data = new FileInformation( base, theFile );
    }
    else
    {
      throw new IOException( "Unknown filesystem object type." );
    }

    data.write( outputStream );

    int statusByte = inputStream.read();
    if( statusByte == -1 )
    {
      throw new IOException( "Unexpected end of stream" );
    }
    else if( statusByte == 0 )
    {
      data.sendData( base, outputStream );
    }
  }

  /**
   * Send the end of tranmission marker.
   *
   * @param stream The stream to get the data from.
   * @return A file information block holding the details of the file.
   */

  public static void sendEnd( InputStream inputStream, OutputStream outputStream )
    throws IOException
  {
    outputStream.write( InformationHandler.END_OF_TRANSMITTION );
    int inByte = inputStream.read();
    while( inByte != -1 && inByte != END_OF_TRANSMITTION )
    {
      inByte = inputStream.read();
    }
  }
}